
local L = AceLibrary('AceLocale-2.2'):new('SharedKoS')

SharedKoS = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceDB-2.0", "AceModuleCore-2.0", "FuBarPlugin-2.0")
local SharedKoS = SharedKoS

if AceLibrary:HasInstance("AceDebug-2.0") then
	AceLibrary("AceDebug-2.0"):embed(SharedKoS)
else
	function SharedKoS:Debug() end
	function SharedKoS:SetDebugging() end
end

function SharedKoS:UpdateVersion(rev_tag, date_tag)
	local rev = tonumber(string.sub(rev_tag, 12, -3))
	if not self.revision or rev > self.revision then
		self.revision = rev
		self.version = "r" .. self.revision
		self.date = string.sub(date_tag, 8, 17)
	end
end

SharedKoS:UpdateVersion("$Revision: 26105 $", "$Date: 2007-01-25 04:04:52 -0500 (Thu, 25 Jan 2007) $")

SharedKoS:RegisterDB('SharedKoSDB')

SharedKoS:RegisterDefaults('realm', {
	scores = {
		['*'] = {
			revision = 0,
			guild = {},
			player = {}
		}
	}
})

function SharedKoS:OnInitialize()
	if not self.db.realm.source then
		self.db.realm.source = UnitName('player')
	end
	self.localsource = self.db.realm.source
	self.localscores = self.db.realm.scores[self.localsource]
end

function SharedKoS:OnEnable()
	self:RegisterEvent('SharedKoS_ScoreChanged', 'CleanupScoreCacheEntry')
end

function SharedKoS:OnDisable()
	for name in core:IterateModules() do
		self:ToggleModuleActive(name, false)
	end
end

-----------------------
-- Database handling --
-----------------------

local TargetTypes = { guild = true, player = true }

function SharedKoS:GetLocalScore(targettype, target)
	AceLibrary:assert(TargetTypes[targettype])
	return self.localscores[targettype][target]
end

function SharedKoS:SetLocalScore(targettype, target, value)
	AceLibrary:assert(TargetTypes[targettype])
	local s = self.localscores
	s[targettype][target] = value
	s.revision = s.revision + 1
	self:TriggerEvent('SharedKoS_LocalScoreChanged', targettype, target, value, s.revision)
	self:TriggerEvent('SharedKoS_ScoreChanged', targettype, target)
end

function SharedKoS:IncLocalScore(targettype, target, value)
	AceLibrary:assert(TargetTypes[targettype])
	return self:SetLocalScore(targettype, target, self.localscores[targettype][target] + value)
end

--------------------------
-- Score cache handling --
--------------------------

local score_cache = {}
do
	function CalculateScore(scores, target)
		local targettype = getmetatable(scores).targettype
		local score = 0
		for source,data in pairs(SharedKoS.db.realm.scores) do
			score = score + (scores[targettype][target] or 0)
		end
		t[name] = score
		return score
	end

	for k in pairs(TargetTypes) do
		local tt = k
		score_cache[k] = setmetatable({}, { __index = CalculateScore, targettype = tt })
	end
end

function SharedKoS:GetScore(targettype, target)
	AceLibrary:assert(TargetTypes[targettype])
	return score_cache[targettype][target]
end

function SharedKoS:CleanupScoreCacheEntry(targettype, target)
	AceLibrary:assert(TargetTypes[targettype])
	score_cache[targettype][target] = nil
end

function SharedKoS:CleanupScoreCache()
	for k in pairs(TargetTypes) do
		cleanupcache(score_cache[k])
	end
end

local function cleanupcache(t) 
	local mt = getmetatable(t)
	setmetatable(t, nil)
	for k in pairs(t) do t[k] = nil end
	setmetatable(t, mt)
end

-----------------------
-- Module definition --
-----------------------

local modProto = SharedKoS.modulePrototype
modProto.core = SharedKoS

if AceLibrary:HasInstance("AceDebug-2.0") then
	SharedKoS:SetModuleMixins("AceDebug-2.0")
else
	modProto.Debug = SharedKoS.Debug
	modProto.SetDebugging = SharedKoS.SetDebugging
end

function modProto:GetAceOptions()
end
