local L = AceLibrary('AceLocale-2.2'):new('SharedKoS')

SharedKoS:UpdateVersion("$Revision: 26105 $", "$Date: 2007-01-25 04:04:52 -0500 (Thu, 25 Jan 2007) $")

local mod = SharedKoS:NewModule('Share', 'AceComm-2.0', 'AceEvent-2.0')

mod.OnComm = {}

function mod:OnInitialize()
	self:SetCommPrefix('SharedKoS')
	self:RegisterMemoizations({'revision', 'guild', 'player'})
	self:SetDebugging(true)
	self:Debug('Initialized');
end

function mod:OnEnable()
	self:ScheduleEvent(function() self:StartSync() end, 5)
	self:RegisterEvent('SharedKoS_LocalScoreChanged', 'LocalScoreChanged')
	self:RegisterComm(self.commPrefix, "GUILD", "OnComm")
	self:Debug('Enabled');
end

function mod:OnDisable()
	self:Debug('Disabled');
end

-----------------------------------------
-- Individual score change propagation --
-----------------------------------------

function mod:LocalScoreChanged(targettype, target, value, revision)
	self:Debug("Propagating local score change: %q:%q=%q (rev %q)", targettype, target, value, revision)
	self:SendCommMessage('GUILD', 'SCORE_UPDATE', self.core.localsource, targettype, target, value, revision)
end

function mod.OnComm:SCORE_UPDATE(prefix, distribution, sender, source, targettype, target, value, revision)
	self:Debug("Received score change: %q:%q=%q (rev %q) for %q from %q", targettype, target, value, revision, source, sender)
	if source ~= self.core.localsource then
		local scores = self.db.realm.scores[source]
		if revision > scores.revision then
			if revision == scores.revision + 1 then
				scores[targettype][target] = value
				scores.revision = revision
				self:TriggerEvent('SharedKoS_ScoreChanged', targettype, target)
			else
				self:SendCommMessage('WHISPER', sender, 'SCORES_QUERY', source)
			end
		end
	end
end

--------------------------
-- Full synchronisation --
--------------------------

local function unpackRevisions(scores, key)
	local nextKey = next(scores, key)
	if nextKey then
		return key, scores[key].revision, unpackRevisions(score, nextKey)
	else
		return key, scores[key].revision
	end
end

function mod:StartSync()
	self:Debug('Requesting sync');
	self:SendCommMessage('GUILD', 'SYNC_REQUEST');
end

function mod.OnComm:SYNC_REQUEST(prefix, distribution, sender)
	if sender ~= UnitName('player') then
		self:Debug('Received sync request from %q', sender);
		self:SendCommMessage('GUILD', 'SYNC_REPLY', unpackRevisions(self.core.db.realm.scores))
	end
end

function mod.OnComm:SYNC_REPLY(prefix, distribution, sender, ...)
	self:Debug('Received sync reply from %q', sender);
	local scores = self.core.db.realm.scores
	local update = false;
	for i = 1,select('#', ...),2 do
		local source, revision = select(i, ...)
		if source ~= self.core.localsource then
			if not scores[source] then
				scores[source] = {
					revision = revision,
					query = sender
				}
				update = true
			else
				local s = scores[source]
				if revision > s[revision] then
					s.revision = revision
					s.query = sender
					update = true
				end
			end
		end
	end
	if update then
		self:ScheduleEvent('QueryUpdates', function() self:QueryUpdates() end, 3)
	end
end

function mod:QueryUpdates()
	self:Debug('Updating')
	for source,scores in pairs(self.core.db.realm.scores) do
		if scores.query then
			self:Debug('- querying update to %q for %q', scores.query, source)
			self:SendCommMessage('WHISPER', scores.query, 'SCORES_QUERY', source)
			scores.query = nil
		end
	end
end

function mod.OnComm:SCORES_QUERY(prefix, distribution, sender, source)
	self:Debug('Sending scores for %s to %s', source, sender)
	self:SendCommMessage('WHISPER', sender, 'SCORES_REPLY', source, self.core.db.realm.scores[source])
end

function mod.OnComm:SCORES_REPLY(prefix, distribution, sender, source, score)
	if source ~= self.core.localsource then
		self:Debug('Receiving scores for %s from %s', source, sender)
		self.core.db.realm.scores[source] = score
		self:ScheduleEvent('CacheCleanup', function() self.core:CleanupScoreCache() end, 1)
	end
end
